@@ -8,11 +8,17 @@ import socket
 
 from typing import Any
 
-STATUS_FILE = 'dmypy.json'
+STATUS_FILE = '.dmypy.json'
 
 
 def receive(sock: socket.socket) -> Any:
-    """Receive JSON data from a socket until EOF."""
+    """Receive JSON data from a socket until EOF.
+
+    Raise a subclass of OSError if there's a socket exception.
+
+    Raise OSError if the data received is not valid JSON or if it is
+    not a dict.
+    """
     bdata = bytearray()
     while True:
         more = sock.recv(100000)
@@ -21,7 +27,10 @@ def receive(sock: socket.socket) -> Any:
         bdata.extend(more)
     if not bdata:
         raise OSError("No data received")
-    data = json.loads(bdata.decode('utf8'))
+    try:
+        data = json.loads(bdata.decode('utf8'))
+    except Exception:
+        raise OSError("Data received is not valid JSON")
     if not isinstance(data, dict):
         raise OSError("Data received is not a dict (%s)" % str(type(data)))
     return data
