@@ -2,7 +2,7 @@ from functools import wraps
 import sys
 
 from typing import (
-    Tuple, Union, TypeVar, Callable, Sequence, Optional, Any, cast, List, Set, overload
+    Tuple, Union, TypeVar, Callable, Sequence, Optional, Any, cast, List, overload
 )
 from mypy.sharedparse import (
     special_function_elide_names, argument_elide_name,
@@ -29,7 +29,6 @@ from mypy.types import (
     TypeOfAny
 )
 from mypy import defaults
-from mypy import experiments
 from mypy import messages
 from mypy.errors import Errors
 from mypy.options import Options
@@ -70,7 +69,7 @@ def parse(source: Union[str, bytes],
           fnam: str,
           module: Optional[str],
           errors: Optional[Errors] = None,
-          options: Options = Options()) -> MypyFile:
+          options: Optional[Options] = None) -> MypyFile:
 
     """Parse a source file, without doing any semantic analysis.
 
@@ -81,6 +80,8 @@ def parse(source: Union[str, bytes],
     if errors is None:
         errors = Errors()
         raise_on_error = True
+    if options is None:
+        options = Options()
     errors.set_file(fnam, module)
     is_stub_file = fnam.endswith('.pyi')
     try:
@@ -396,8 +397,7 @@ class ASTConverter(ast3.NodeTransformer):
                        self.as_required_block(n.body, n.lineno),
                        func_type)
         if is_coroutine:
-            # A coroutine is also a generator, mostly for internal reasons.
-            func_def.is_generator = func_def.is_coroutine = True
+            func_def.is_coroutine = True
         if func_type is not None:
             func_type.definition = func_def
             func_type.line = n.lineno
@@ -958,8 +958,12 @@ class ASTConverter(ast3.NodeTransformer):
 
     # List(expr* elts, expr_context ctx)
     @with_line
-    def visit_List(self, n: ast3.List) -> ListExpr:
-        return ListExpr([self.visit(e) for e in n.elts])
+    def visit_List(self, n: ast3.List) -> Union[ListExpr, TupleExpr]:
+        expr_list = [self.visit(e) for e in n.elts]  # type: List[Expression]
+        if isinstance(n.ctx, ast3.Store):
+            # [x, y] = z and (x, y) = z means exactly the same thing
+            return TupleExpr(expr_list)
+        return ListExpr(expr_list)
 
     # Tuple(expr* elts, expr_context ctx)
     @with_line
@@ -1153,6 +1157,7 @@ class TypeConverter(ast3.NodeTransformer):
 
     # List(expr* elts, expr_context ctx)
     def visit_List(self, n: ast3.List) -> Type:
+        assert isinstance(n.ctx, ast3.Load)
         return self.translate_argument_list(n.elts)
 
 
